#include <SPI.h>
#include <Ethernet.h>
#include <EthernetUdp.h>

int ledpin = 7;

unsigned int local_port = 33333;
unsigned int remote_Port = 9999;
const char * server = "your-domain.com";
String device_name = "Arduino";
byte mac[] = {0x90, 0xA2, 0xDA, 0x0D, 0x5C, 0x18};

char packetBuffer[UDP_TX_PACKET_MAX_SIZE];
long timer1;
EthernetUDP Udp;

void setup() {
  pinMode(ledpin, OUTPUT);
  Serial.begin(9600);
  Serial.println("Initialize Ethernet Shield with DHCP...");
  if (Ethernet.begin(mac) == 0) {
    Serial.println("Error: IP address was not assigned by DHCP");
    if (Ethernet.hardwareStatus() == EthernetNoHardware) {
      Serial.println("Error: Ethernet shield was not found");
    } else if (Ethernet.linkStatus() == LinkOFF) {
      Serial.println("Ethernet cable is not connected");
    } while (true) {
      delay(1);
    }
  }
  Serial.print("Local IP: ");
  Serial.print(Ethernet.localIP());
  Serial.print(" PORT: ");
  Serial.println(local_port);
  Udp.begin(local_port);
  delay(2000);
  sendudp("Keep-Alive");
}

void loop() {

  if ((timer1 + (30 * 1000)) <= millis()) { //Send a UDP packet every 30 seconds to keep port open
    sendudp("Keep-Alive");
    timer1 = millis();
  }

  int packetSize = Udp.parsePacket();

  if (packetSize) {

    Serial.print("Received packet! (Size: ");
    Serial.print(packetSize);
    Serial.println(")");
    Serial.print("From ");
    Serial.print(Udp.remoteIP());
    Serial.print(":");
    Serial.println(Udp.remotePort());

    Udp.read(packetBuffer, UDP_TX_PACKET_MAX_SIZE);
    String received_string(packetBuffer);

    Serial.print("<- Received String: ");
    Serial.println(received_string);
    Serial.println("");
    memset(&packetBuffer[0], 0, sizeof(packetBuffer)); //Clear buffer

    if (received_string.indexOf("LED=1") > 0) {
      Serial.println("TURN LED ON!");
      delay(10);
      digitalWrite(ledpin, HIGH);
      sendudp("DONE");
    }

    if (received_string.indexOf("LED=0") > 0) {
      Serial.println("TURN LED OFF!");
      delay(10);
      digitalWrite(ledpin, LOW);
      sendudp("DONE");
    }

    if (received_string.indexOf("INFO") > 0) {
      Serial.println("INFO COMMAND RECEIVED!");
      sendinfoudp();
    }
  }
}

void sendudp(String packet) {
  Udp.beginPacket(server, remote_Port);
  Udp.print(";");
  Udp.print(device_name);
  Udp.print(";");
  Udp.print(packet);
  Udp.print(";");
  Udp.endPacket();

  Serial.print("-> Send UDP packet to server: ");
  Serial.println(packet);
  Serial.print("-> RAW:  ");
  Serial.print(";");
  Serial.print(device_name);
  Serial.print(";");
  Serial.print(packet);
  Serial.println(";");
  Serial.println("");
  delay(2);
}

void sendinfoudp() {
  Udp.beginPacket(server, remote_Port);
  Udp.print(";");
  Udp.print(device_name);
  Udp.print(";");
  Udp.print("Arduino-IP: ");
  Udp.print(Ethernet.localIP());
  Udp.print(" Uptime: ");
  Udp.print(millis() / 1000);
  Udp.print(" sec");
  Udp.print(";");
  Udp.endPacket();
  delay(2);
}
